// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import assert from 'assert';
import BigInt from 'big-integer';
import C25519Point from '../src/c25519Point';
import utils from '../src/utils';
import ElGamalKey from '../src/elGamalKey';
import TObliviousFlow from '../src/tobliviousFlow';

var threshold = 3;
var user = 'admin';
var pass = "123456";
var k = BigInt("eadd47c4185363dd86a4e99064ee9fcfb4116b1011a9a33dbb658eeeefed6aa", 16);
var urls = [...Array(threshold)].map((_, i) => 'http://127.0.0.1:500' + (i + 1));
var flow = new TObliviousFlow(urls, user);

(async () => {
    await main();
})();

async function main() {
    try {
        await flow.signIn(k, threshold);
        var keyTag = await flow.logIn(pass);

        var g = C25519Point.fromString(pass);
        var [secKey, pubKey] = ElGamalKey.seed(g.times(k).toArray());
        
        assert.equal(keyTag.priv.key.eq(secKey.key), true);
        assert.equal(keyTag.pub.key.eq(pubKey.key), true);
    } catch (error) {
        console.log(error);
    }
}
