// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import EcScalar from './ecScalar';

const size = 32;

export default class EcScalarListSerializer {
    
    /** @param {EcScalar[]} numbers */
    static serializeStr(numbers) {
        return Buffer.from(this.serialize(numbers)).toString('base64');
    }
    
    /**
     * @param {EcScalar[]} numbers
     * @returns {Uint8Array}
     */
    static serialize(numbers)
    {
        var data = new Uint8Array(numbers.length * size + 1);

        var i = 1;
        data[0] = numbers.length;
        for (const num of numbers) {
            data.set(num.toArray(), i);
            i += size;
        }

        return data;
    }

    /**
     * @param {Uint8Array|string} data
     * @returns {EcScalar[]}
     */
    static deserialize(data) {
        var bytes = typeof data === 'string'? Buffer.from(data, 'base64') : data;
        if (bytes === null || bytes.length < 1 || (bytes[0] === 0 && bytes.length > 1))
            throw new Error("Format Error");

        var len = bytes[0];
        if (len === 0)
            return [];
        
        var numbers = new Array(len);
        for (let i = 0; i < len; i++) {
            numbers[i] = EcScalar.fromArray(bytes.slice(i * size + 1).slice(0, size));
        }
        return numbers;
    }
}
