import assert from 'assert';
import elGamal from '../src/elGamal.js';
import ElGamalKey from '../src/elGamalKey.js';
import utils, { constants } from '../src/utils.js';

describe('ElGamal', function () {
    const bits = 32;
    const msg = "Hi, it's me, Mario";

    describe('encrypt()', function () {
        describe('decrypt()', function () {
            it('should reconstruct message', function () {
                for (let i = 0; i < 50; i++) {
                    const [prv, pub] = ElGamalKey.generate(bits);
                    const cypher = elGamal.encrypt(msg, pub);
                    assert.equal(msg, elGamal.decrypt(cypher, prv));
                }
            });
        });
    });

    describe('toString()', function () {
        describe('from()', function () {
            it('should serialize and deserialize the key', function () {
                for (let i = 0; i < 20; i++) {
                    var [prv, pub] = ElGamalKey.generate(32);
                    const prvCopy = ElGamalKey.from(prv.toString());

                    assert.equal(prv.p.toString(16), prvCopy.p.toString(16));
                    assert.equal(prv.q.toString(16), prvCopy.q.toString(16));
                    assert.equal(prv.g.toString(16), prvCopy.g.toString(16));
                    assert.equal(prv.key.toString(16), prvCopy.key.toString(16));
                    assert.equal(prv.id, prvCopy.id);
                    assert.equal(prv.privated, prvCopy.privated);

                    pub.id = utils.random(constants.one, pub.p.minus(constants.one));
                    const pubCopy = ElGamalKey.from(pub.toString());

                    assert.equal(pub.id.toString(16), pubCopy.id.toString(16));
                    assert.equal(pub.privated, pubCopy.privated);
                }
            });
        });
    });

    describe('share()', function () {
        const number = 5;
        const threshold = 3;

        describe('combine()', function () {
            it('reconstruct key shares', function () {
                for (let i = 0; i < 25; i++) {
                    const [prv] = ElGamalKey.generate(bits);
                    const sharedKeys = prv.share(number, threshold).slice(-threshold)
                    const joinedKey = ElGamalKey.combine(sharedKeys)
                    assert.equal(prv.toString(), joinedKey.toString())
                }
            });
        });

        describe('combinePartial()', function () {
            it('reconstruct partial descriptions', function () {
                for (let i = 0; i < 25; i++) {
                    const [prv, pub] = ElGamalKey.generate(bits);
                    const cypher = elGamal.encrypt(msg, pub);

                    const sharedKeys = prv.share(number, threshold).slice(-threshold)
                    const partialCypher = sharedKeys.map(sh => elGamal.decryptPartial(cypher, sh));

                    var joinedMsg = elGamal.combinePartial(partialCypher, pub);
                    assert.equal(msg, joinedMsg)
                }
            });
        });
    });
});
